@@ -4,12 +4,12 @@ use std::str::FromStr;
 use ruff_diagnostics::{Diagnostic, Edit, Fix, FixAvailability, Violation};
 use ruff_macros::{derive_message_formats, violation};
 use ruff_python_ast::whitespace::indentation;
-use ruff_python_ast::{self as ast, Expr};
+use ruff_python_ast::{self as ast, AnyStringKind, Expr};
 use ruff_python_codegen::Stylist;
 use ruff_python_literal::cformat::{
     CConversionFlags, CFormatPart, CFormatPrecision, CFormatQuantity, CFormatString,
 };
-use ruff_python_parser::{lexer, AsMode, StringKind, Tok};
+use ruff_python_parser::{lexer, AsMode, Tok};
 use ruff_python_stdlib::identifiers::is_identifier;
 use ruff_source_file::Locator;
 use ruff_text_size::{Ranged, TextRange};
@@ -19,7 +19,8 @@ use crate::checkers::ast::Checker;
 use crate::rules::pyupgrade::helpers::curly_escape;
 
 /// ## What it does
-/// Checks for `printf`-style string formatting.
+/// Checks for `printf`-style string formatting, and offers to replace it with
+/// `str.format` calls.
 ///
 /// ## Why is this bad?
 /// `printf`-style string formatting has a number of quirks, and leads to less
@@ -27,30 +28,6 @@ use crate::rules::pyupgrade::helpers::curly_escape;
 /// the newer `str.format` and f-strings constructs over `printf`-style string
 /// formatting.
 ///
-/// ## Known problems
-/// This rule is unable to detect cases in which the format string contains
-/// a single, generic format specifier (e.g. `%s`), and the right-hand side
-/// is an ambiguous expression.
-///
-/// For example, given:
-/// ```python
-/// "%s" % value
-/// ```
-///
-/// `value` could be a single-element tuple, _or_ it could be a single value.
-/// Both of these would resolve to the same formatted string when using
-/// `printf`-style formatting, but _not_ when using f-strings:
-///
-/// ```python
-/// value = 1
-/// print("%s" % value)  # "1"
-/// print("{}".format(value))  # "1"
-///
-/// value = (1,)
-/// print("%s" % value)  # "1"
-/// print("{}".format(value))  # "(1,)"
-/// ```
-///
 /// ## Example
 /// ```python
 /// "%s, %s" % ("Hello", "World")  # "Hello, World"
@@ -61,6 +38,31 @@ use crate::rules::pyupgrade::helpers::curly_escape;
 /// "{}, {}".format("Hello", "World")  # "Hello, World"
 /// ```
 ///
+/// ## Fix safety
+/// In cases where the format string contains a single generic format specifier
+/// (e.g. `%s`), and the right-hand side is an ambiguous expression,
+/// we cannot offer a safe fix.
+///
+/// For example, given:
+/// ```python
+/// "%s" % val
+/// ```
+///
+/// `val` could be a single-element tuple, _or_ a single value (not
+/// contained in a tuple). Both of these would resolve to the same
+/// formatted string when using `printf`-style formatting, but
+/// resolve differently when using f-strings:
+///
+/// ```python
+/// val = 1
+/// print("%s" % val)  # "1"
+/// print("{}".format(val))  # "1"
+///
+/// val = (1,)
+/// print("%s" % val)  # "1"
+/// print("{}".format(val))  # "(1,)"
+/// ```
+///
 /// ## References
 /// - [Python documentation: `printf`-style String Formatting](https://docs.python.org/3/library/stdtypes.html#old-string-formatting)
 /// - [Python documentation: `str.format`](https://docs.python.org/3/library/stdtypes.html#str.format)
@@ -260,9 +262,7 @@ fn clean_params_dictionary(right: &Expr, locator: &Locator, stylist: &Stylist) -
         }
         contents.push('(');
         if is_multi_line {
-            let Some(indent) = indent else {
-                return None;
-            };
+            let indent = indent?;
 
             for item in &arguments {
                 contents.push_str(stylist.line_ending().as_str());
@@ -352,7 +352,7 @@ fn convertible(format_string: &CFormatString, params: &Expr) -> bool {
 /// UP031
 pub(crate) fn printf_string_formatting(checker: &mut Checker, expr: &Expr, right: &Expr) {
     // Grab each string segment (in case there's an implicit concatenation).
-    let mut strings: Vec<(TextRange, StringKind)> = vec![];
+    let mut strings: Vec<(TextRange, AnyStringKind)> = vec![];
     let mut extension = None;
     for (tok, range) in lexer::lex_starts_at(
         checker.locator().slice(expr),
@@ -436,7 +436,8 @@ pub(crate) fn printf_string_formatting(checker: &mut Checker, expr: &Expr, right
                 // print("%s" % x)
                 // print("{}".format(x))
                 // ```
-                return;
+                // So we offer an unsafe fix:
+                Cow::Owned(format!("({})", checker.locator().slice(right)))
             }
         }
         Expr::Tuple(_) => clean_params_tuple(right, checker.locator()),
