@@ -1,6 +1,7 @@
 use crate::{server::schedule::Task, session::Session};
 use lsp_server as server;
 
+mod diagnostics;
 mod notifications;
 mod requests;
 mod traits;
@@ -16,8 +17,8 @@ use super::{client::Responder, schedule::BackgroundSchedule, Result};
 /// given the parameter type used by the implementer.
 macro_rules! define_document_url {
     ($params:ident: &$p:ty) => {
-        fn document_url($params: &$p) -> &lsp_types::Url {
-            &$params.text_document.uri
+        fn document_url($params: &$p) -> std::borrow::Cow<lsp_types::Url> {
+            std::borrow::Cow::Borrowed(&$params.text_document.uri)
         }
     };
 }
@@ -28,22 +29,29 @@ pub(super) fn request<'a>(req: server::Request) -> Task<'a> {
     let id = req.id.clone();
 
     match req.method.as_str() {
-        request::CodeAction::METHOD => background_request_task::<request::CodeAction>(
+        request::CodeActions::METHOD => background_request_task::<request::CodeActions>(
             req,
             BackgroundSchedule::LatencySensitive,
         ),
+        request::CodeActionResolve::METHOD => {
+            background_request_task::<request::CodeActionResolve>(req, BackgroundSchedule::Worker)
+        }
         request::DocumentDiagnostic::METHOD => {
             background_request_task::<request::DocumentDiagnostic>(
                 req,
                 BackgroundSchedule::LatencySensitive,
             )
         }
+        request::ExecuteCommand::METHOD => local_request_task::<request::ExecuteCommand>(req),
         request::Format::METHOD => {
             background_request_task::<request::Format>(req, BackgroundSchedule::Fmt)
         }
         request::FormatRange::METHOD => {
             background_request_task::<request::FormatRange>(req, BackgroundSchedule::Fmt)
         }
+        request::Hover::METHOD => {
+            background_request_task::<request::Hover>(req, BackgroundSchedule::Worker)
+        }
         method => {
             tracing::warn!("Received request {method} which does not have a handler");
             return Task::nothing();
@@ -51,6 +59,9 @@ pub(super) fn request<'a>(req: server::Request) -> Task<'a> {
     }
     .unwrap_or_else(|err| {
         tracing::error!("Encountered error when routing request with ID {id}: {err}");
+        show_err_msg!(
+            "Ruff failed to handle a request from the editor. Check the logs for more details."
+        );
         let result: Result<()> = Err(err);
         Task::immediate(id, result)
     })
@@ -80,17 +91,17 @@ pub(super) fn notification<'a>(notif: server::Notification) -> Task<'a> {
     }
     .unwrap_or_else(|err| {
         tracing::error!("Encountered error when routing notification: {err}");
+        show_err_msg!("Ruff failed to handle a notification from the editor. Check the logs for more details.");
         Task::nothing()
     })
 }
 
-#[allow(dead_code)]
 fn local_request_task<'a, R: traits::SyncRequestHandler>(
     req: server::Request,
 ) -> super::Result<Task<'a>> {
     let (id, params) = cast_request::<R>(req)?;
-    Ok(Task::local(|session, notifier, responder| {
-        let result = R::run(session, notifier, params);
+    Ok(Task::local(|session, notifier, requester, responder| {
+        let result = R::run(session, notifier, requester, params);
         respond::<R>(id, result, &responder);
     }))
 }
@@ -102,7 +113,7 @@ fn background_request_task<'a, R: traits::BackgroundDocumentRequestHandler>(
     let (id, params) = cast_request::<R>(req)?;
     Ok(Task::background(schedule, move |session: &Session| {
         // TODO(jane): we should log an error if we can't take a snapshot.
-        let Some(snapshot) = session.take_snapshot(R::document_url(&params)) else {
+        let Some(snapshot) = session.take_snapshot(&R::document_url(&params)) else {
             return Box::new(|_, _| {});
         };
         Box::new(move |notifier, responder| {
@@ -116,9 +127,10 @@ fn local_notification_task<'a, N: traits::SyncNotificationHandler>(
     notif: server::Notification,
 ) -> super::Result<Task<'a>> {
     let (id, params) = cast_notification::<N>(notif)?;
-    Ok(Task::local(move |session, notifier, _| {
-        if let Err(err) = N::run(session, notifier, params) {
+    Ok(Task::local(move |session, notifier, requester, _| {
+        if let Err(err) = N::run(session, notifier, requester, params) {
             tracing::error!("An error occurred while running {id}: {err}");
+            show_err_msg!("Ruff encountered a problem. Check the logs for more details.");
         }
     }))
 }
@@ -131,12 +143,13 @@ fn background_notification_thread<'a, N: traits::BackgroundDocumentNotificationH
     let (id, params) = cast_notification::<N>(req)?;
     Ok(Task::background(schedule, move |session: &Session| {
         // TODO(jane): we should log an error if we can't take a snapshot.
-        let Some(snapshot) = session.take_snapshot(N::document_url(&params)) else {
+        let Some(snapshot) = session.take_snapshot(&N::document_url(&params)) else {
             return Box::new(|_, _| {});
         };
         Box::new(move |notifier, _| {
             if let Err(err) = N::run_with_snapshot(snapshot, notifier, params) {
                 tracing::error!("An error occurred while running {id}: {err}");
+                show_err_msg!("Ruff encountered a problem. Check the logs for more details.");
             }
         })
     }))
@@ -179,6 +192,10 @@ fn respond<Req>(
 ) where
     Req: traits::RequestHandler,
 {
+    if let Err(err) = &result {
+        tracing::error!("An error occurred with result ID {id}: {err}");
+        show_err_msg!("Ruff encountered a problem. Check the logs for more details.");
+    }
     if let Err(err) = responder.respond(id, result) {
         tracing::error!("Failed to send response: {err}");
     }
